import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:get_it/get_it.dart';
import 'package:okane/database/sqlite.dart';
import 'package:okane/i18n/strings.g.dart';
import 'package:okane/ui/pages/budgets/add_budget_item.dart';
import 'package:okane/ui/state/core.dart';
import 'package:okane/ui/utils.dart';
import 'package:okane/ui/widgets/piechart_card.dart';

class BudgetDetailsPage extends StatelessWidget {
  final bool isPage;

  const BudgetDetailsPage({this.isPage = false, super.key});

  static MaterialPageRoute<void> get mobileRoute =>
      MaterialPageRoute(builder: (_) => BudgetDetailsPage(isPage: true));

  void _addBudgetItem(BuildContext context, CoreState state) {
    showDialogOrModal(
      context: context,
      builder:
          (_) => AddBudgetItemPopup(
            budget: state.activeBudget!,
            items: state.budgetItems,
            onDone: () {
              Navigator.of(context).pop();
            },
          ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      body: ListView(
        children: [
          if (isPage)
            SizedBox(
              height: 50,
              child: Row(
                children: [
                  IconButton(
                    icon: Icon(Icons.arrow_back),
                    onPressed: () {
                      Navigator.of(context).pop();
                    },
                  ),
                ],
              ),
            ),

          BlocBuilder<CoreCubit, CoreState>(
            builder: (context, state) {
              if (state.activeBudget == null) {
                return Text(t.pages.budgets.details.noBudgetSelected);
              }

              if (state.budgetItems.isEmpty) {
                return Row(
                  children: [
                    Text(t.pages.budgets.details.noBudgetItems),
                    Padding(
                      padding: EdgeInsets.only(left: 16),
                      child: IconButton(
                        onPressed: () => _addBudgetItem(context, state),
                        icon: Icon(Icons.add),
                      ),
                    ),
                  ],
                );
              }

              final bloc = GetIt.I.get<CoreCubit>();
              final today = DateTime.now();
              final db = GetIt.I.get<OkaneDatabase>();
              return FutureBuilder(
                future: db.transactionsDao.getTransactionsInTimeframe(
                  bloc.activeAccount!,
                  today,
                  TransactionQueryDateOption.thisMonth,
                ),
                builder: (context, snapshot) {
                  final daysLeft = switch (state.activeBudget!.budget.period) {
                    BudgetPeriod.month =>
                      monthEnding(today).difference(today).inDays,
                  };

                  if (!snapshot.hasData) {
                    return Column(
                      mainAxisSize: MainAxisSize.min,
                      children: [
                        Text(
                          t.pages.budgets.details.budgetItems,
                          style: Theme.of(context).textTheme.titleMedium,
                        ),
                        ListView.builder(
                          shrinkWrap: true,
                          itemCount: state.budgetItems.length,
                          itemBuilder: (context, index) {
                            final item = state.budgetItems.elementAt(index);
                            final amount = formatCurrency(item.item.amount);
                            return ListTile(
                              title: Text(
                                t.pages.budgets.details.items.title(
                                  // TODO
                                  name: "lol",
                                  //name: item.expenseCategory.value!.name,
                                  amount: amount,
                                ),
                              ),
                              subtitle: Text(
                                t.pages.budgets.details.items.loading,
                              ),
                            );
                          },
                        ),
                      ],
                    );
                  }

                  final categories =
                      state.budgetItems
                          // TODO
                          //.map((i) => i.expenseCategory.value!.name)
                          .map((i) => "lol")
                          .toList();
                  final spending = <String, double>{};
                  for (final t in snapshot.data!) {
                    String categoryName;
                    if (!categories.contains(t.expenseCategory?.name)) {
                      if (!state.activeBudget!.budget.includeOtherSpendings) {
                        continue;
                      }

                      categoryName = "Other";
                    } else {
                      categoryName = t.expenseCategory!.name;
                    }

                    spending.update(
                      categoryName,
                      (value) => value + t.transaction.amount,
                      ifAbsent: () => t.transaction.amount,
                    );
                  }

                  final totalSpent =
                      spending.isEmpty
                          ? 0
                          : spending.values.reduce((acc, val) => acc + val);
                  final budgetTotal = state.budgetItems
                      .map((i) => i.item.amount)
                      .reduce((acc, val) => acc + val);
                  return Column(
                    mainAxisSize: MainAxisSize.min,
                    crossAxisAlignment: CrossAxisAlignment.start,
                    children: [
                      SizedBox(
                        height: 100,
                        child: ListView(
                          scrollDirection: Axis.horizontal,
                          children: [
                            Padding(
                              padding: EdgeInsets.all(8),
                              child: SizedBox(
                                height: 100,
                                width: 150,
                                child: Card(
                                  child: Column(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.center,
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    children: [
                                      Text(
                                        t.pages.budgets.details.daysLeft,
                                        textAlign: TextAlign.center,
                                        style:
                                            Theme.of(
                                              context,
                                            ).textTheme.titleLarge,
                                      ),
                                      Text(
                                        daysLeft.toString(),
                                        textAlign: TextAlign.center,
                                        style:
                                            Theme.of(
                                              context,
                                            ).textTheme.bodyLarge,
                                      ),
                                    ],
                                  ),
                                ),
                              ),
                            ),
                            Padding(
                              padding: EdgeInsets.all(8),
                              child: SizedBox(
                                height: 100,
                                width: 150,
                                child: Card(
                                  child: Column(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.center,
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    children: [
                                      Text(
                                        t.pages.budgets.details.budgetLeft,
                                        textAlign: TextAlign.center,
                                        style:
                                            Theme.of(
                                              context,
                                            ).textTheme.titleLarge,
                                      ),
                                      Text(
                                        formatCurrency(
                                          budgetTotal + totalSpent,
                                        ),
                                        textAlign: TextAlign.center,
                                        style:
                                            Theme.of(
                                              context,
                                            ).textTheme.bodyLarge,
                                      ),
                                    ],
                                  ),
                                ),
                              ),
                            ),
                            Padding(
                              padding: EdgeInsets.all(8),
                              child: SizedBox(
                                height: 100,
                                width: 150,
                                child: Card(
                                  child: Column(
                                    crossAxisAlignment:
                                        CrossAxisAlignment.center,
                                    mainAxisAlignment: MainAxisAlignment.center,
                                    children: [
                                      Text(
                                        t.pages.budgets.details.totalBudget,
                                        textAlign: TextAlign.center,
                                        style:
                                            Theme.of(
                                              context,
                                            ).textTheme.titleLarge,
                                      ),
                                      Text(
                                        formatCurrency(budgetTotal),
                                        textAlign: TextAlign.center,
                                        style:
                                            Theme.of(
                                              context,
                                            ).textTheme.bodyLarge,
                                      ),
                                    ],
                                  ),
                                ),
                              ),
                            ),
                          ],
                        ),
                      ),

                      Wrap(
                        children: [
                          Padding(
                            padding: EdgeInsets.symmetric(horizontal: 8),
                            child: PieChartCard(
                              fallbackText: "",
                              valueConverter: formatCurrency,
                              items:
                                  state.budgetItems
                                      .map(
                                        (i) => (
                                          title: i.expenseCategory.name,
                                          value: i.item.amount,
                                          color: colorHash(
                                            i.expenseCategory.name,
                                          ),
                                        ),
                                      )
                                      .toList(),
                              titleText:
                                  t.pages.budgets.details.budgetBreakdown.title,
                            ),
                          ),

                          Padding(
                            padding: EdgeInsets.symmetric(horizontal: 8),
                            child: PieChartCard(
                              fallbackText:
                                  t
                                      .pages
                                      .budgets
                                      .details
                                      .budgetBreakdown
                                      .noSpendingAvailable,
                              valueConverter: formatCurrency,
                              items:
                                  spending.entries
                                      .map(
                                        (e) => (
                                          title: e.key,
                                          value: e.value.abs(),
                                          color: colorHash(e.key),
                                        ),
                                      )
                                      .toList(),
                              titleText:
                                  t
                                      .pages
                                      .budgets
                                      .details
                                      .spendingBreakdown
                                      .title,
                            ),
                          ),
                        ],
                      ),

                      Padding(
                        padding: EdgeInsets.all(8),
                        child: Row(
                          children: [
                            Text(
                              t.pages.budgets.details.budgetItems,
                              style: Theme.of(context).textTheme.titleMedium,
                            ),
                            Padding(
                              padding: EdgeInsets.only(left: 16),
                              child: IconButton(
                                icon: Icon(Icons.add),
                                onPressed: () => _addBudgetItem(context, state),
                              ),
                            ),
                          ],
                        ),
                      ),
                      Padding(
                        padding: EdgeInsets.all(8),
                        child: ListView.builder(
                          shrinkWrap: true,
                          itemCount: state.budgetItems.length,
                          itemBuilder: (context, index) {
                            final item = state.budgetItems.elementAt(index);
                            final amount = formatCurrency(item.item.amount);
                            final spent = spending[item.expenseCategory.name];
                            final left =
                                spent == null
                                    ? item.item.amount
                                    : item.item.amount + spent;
                            final subtitleText =
                                left < 0
                                    ? t.pages.budgets.details.items.over(
                                      amount: formatCurrency(left),
                                    )
                                    : t.pages.budgets.details.items.remaining(
                                      amount: formatCurrency(left),
                                    );
                            return ListTile(
                              title: Text(
                                "${item.expenseCategory.name} ($amount)",
                              ),
                              subtitle: Text(
                                subtitleText,
                                style: TextStyle(
                                  color: left < 0 ? Colors.red : Colors.green,
                                ),
                              ),
                              trailing: IconButton(
                                icon: Icon(Icons.delete),
                                onPressed: () {},
                              ),
                            );
                          },
                        ),
                      ),
                    ],
                  );
                },
              );
            },
          ),
        ],
      ),
    );
  }
}
